import torch
from .deployable_module import DeployableModule


def compile(
    torch_module: torch.nn.Module, *, backend="nexfort", options=None
) -> DeployableModule:
    from .backends.registry import lookup_backend

    backend = lookup_backend(backend)
    model = backend(torch_module, options=options)
    return model


def oneflow_compile(torch_module: torch.nn.Module, *, options=None) -> DeployableModule:
    return compile(torch_module, backend="oneflow", options=options)



import numpy as np
from onediff.infer_compiler import oneflow_compile
from onediff.infer_compiler.transform import register
import torch
import torch.nn as nn
import oneflow as flow


class MyModule(nn.Module):
    def __init__(self):
        super().__init__()
        self.linears = nn.ModuleList([nn.Linear(10, 10) for i in range(10)])

    def forward(self, x):
        for i, l in enumerate(self.linears):
            x = self.linears[i // 2](x) + l(x)
        return x


class MyModuleOneflow(flow.nn.Module):
    def __init__(self):
        super().__init__()
        self.linears = flow.nn.ModuleList([flow.nn.Linear(10, 10) for i in range(10)])

    def forward(self, x):
        for i, l in enumerate(self.linears):
            x = self.linears[i // 2](x) + l(x)
        return x


register(torch2oflow_class_map={MyModule: MyModuleOneflow})

m = MyModule().to("cuda")
x = torch.randn(2, 10).to("cuda")
y_torch = m(x)

m = oneflow_compile(m)
y_oneflow = m(x)

assert np.allclose(y_torch.detach().cpu(), y_oneflow.detach().cpu(), 1e-03, 1e-03)

from onediff.infer_compiler.oneflow.dual_module import DualModule, DualModuleList

assert isinstance(m.linears, DualModuleList)

x = getattr(m.linears, "1")
assert isinstance(x, DualModule)

x.bias = None
setattr(m.linears, "2", x)

assert m.linears[2].bias is None
assert m.linears._torch_modules[2].bias is None
assert m.linears._oneflow_modules[2].bias is None

m.linears[3] = x

assert m.linears[3].bias is None
assert m.linears._torch_modules[3].bias is None
assert m.linears._oneflow_modules[3].bias is None



import importlib
import os
import unittest

import oneflow as flow
import torch
from torch import nn

from onediff.infer_compiler import oneflow_compile
from onediff.infer_compiler.transform import register
from onediff.infer_compiler.utils import is_community_version

is_community = is_community_version()
onediff_quant_spec = importlib.util.find_spec("onediff_quant")
if is_community or onediff_quant_spec is None:
    print(f"{is_community=} {onediff_quant_spec=}")
    exit(0)

from onediff_quant.quantization import (
    OfflineQuantModule,
    OnlineQuantModule,
    QuantizationConfig,
    create_quantization_calculator,
)


# Define the model
class SimpleModel(nn.Module):
    def __init__(self):
        super(SimpleModel, self).__init__()
        # Two convolutional layers
        self.conv1 = nn.Conv2d(
            in_channels=3, out_channels=16, kernel_size=3, stride=1, padding=1
        )
        self.conv2 = nn.Conv2d(
            in_channels=16, out_channels=32, kernel_size=3, stride=1, padding=1
        )
        # Fully connected layer
        self.fc = nn.Linear(
            32 * 32 * 32, 4
        )  # Input channels are 32*32*32, output 4 classes

    def forward(self, x):
        x = torch.relu(self.conv1(x))
        x = torch.relu(self.conv2(x))
        x = x.view(x.size(0), -1)  # Flatten the tensor into one dimension
        x = self.fc(x)
        return x


class SimpleModel_OF(flow.nn.Module):
    def __init__(self):
        super(SimpleModel, self).__init__()
        # Two convolutional layers
        self.conv1 = flow.nn.Conv2d(
            in_channels=3, out_channels=16, kernel_size=3, stride=1, padding=1
        )
        self.conv2 = flow.nn.Conv2d(
            in_channels=16, out_channels=32, kernel_size=3, stride=1, padding=1
        )
        # Fully connected layer
        self.fc = flow.nn.Linear(
            32 * 32 * 32, 4
        )  # Input channels are 32*32*32, output 10 classes

    def forward(self, x):
        x = flow.relu(self.conv1(x))
        x = flow.relu(self.conv2(x))
        x = x.view(x.size(0), -1)  # Flatten the tensor into one dimension
        x = self.fc(x)
        return x


register(torch2oflow_class_map={SimpleModel: SimpleModel_OF})

# Configure quantization
config = QuantizationConfig.from_settings(
    quantize_conv=True,
    quantize_linear=True,
    cache_dir="cache_dir",
    plot_calibrate_info=False,
)
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
model = SimpleModel().to(device).to(torch.float16)
input_data = torch.randn(1, 3, 32, 32, dtype=torch.float16).to(
    device
)  # Input data size is [batch_size, channels, height, width]
seed = 1
calculator = create_quantization_calculator(model, config)
torch.manual_seed(seed)
standard_output = model(input_data)


class TestOnlineQuantModule(unittest.TestCase):
    def setUp(self):
        self.module = OnlineQuantModule(calculator, inplace=False)

    def test_quantize_with_calibration(self):
        quantized_model, info = self.module.quantize_with_calibration(input_data)
        status = self.module.collect_quantization_status(model, info)
        assert (
            status["quantized_conv_count"] == 2
            and status["quantized_linear_count"] == 1
        )
        compiled_model = oneflow_compile(quantized_model)
        torch.manual_seed(seed)
        quantized_output = compiled_model(input_data)
        # print(f'{quantized_output=} \n{standard_output=}')
        self.assertTrue(torch.allclose(standard_output, quantized_output, 1e4, 1e4))


class TestOfflineQuantModule(unittest.TestCase):
    def setUp(self):
        self.module = OfflineQuantModule(calculator, inplace=False)

    def test_quantize_with_calibration(self):
        quantized_model = self.module.quantize_with_calibration(input_data)[0]
        file_path = os.path.join(config.cache_dir, "quantized_model.pt")
        self.module.save(quantized_model, file_path)

        quantized_model = OfflineQuantModule(None).load(file_path=file_path)
        compiled_model = oneflow_compile(quantized_model)

        torch.manual_seed(seed)
        quantized_output = compiled_model(input_data)
        self.assertTrue(torch.allclose(standard_output, quantized_output, 1e4, 1e4))


if __name__ == "__main__":
    unittest.main()



import oneflow as flow

from ..transform.manager import transform_mgr
from ..transform.builtin_transform import reverse_proxy_class
from ..utils.log_utils import logger
from ..utils.cost_util import cost_cnt


class OneflowGraph(flow.nn.Graph):
    @flow.nn.Graph.with_dynamic_input_shape()
    def __init__(self, model):
        super().__init__(enable_get_runtime_state_dict=True)
        self.model = model
        logger.info(f"Building a graph for {model.__class__.__name__} ...")
        # self.config.enable_cudnn_conv_heuristic_search_algo(False)
        self.config.allow_fuse_add_to_output(True)

    def build(self, *args, **kwargs):
        return self.model(*args, **kwargs)

    @cost_cnt(transform_mgr.debug_mode)
    def load_graph(self, file_path, device=None, run_warmup=True):
        state_dict = flow.load(file_path)
        self.graph_state_dict = state_dict  # used for OneflowGraph.save_graph
        if device is not None:
            state_dict = flow.nn.Graph.runtime_state_dict_to(state_dict, device)
        self.load_runtime_state_dict(state_dict, warmup_with_run=run_warmup)

    @cost_cnt(transform_mgr.debug_mode)
    def save_graph(self, file_path):
        if hasattr(self, "graph_state_dict"):
            flow.save(self.graph_state_dict, file_path)
            return

        state_dict = self.runtime_state_dict()

        import oneflow.framework.args_tree as args_tree

        def disabled_dataclass(value):
            return False

        original_is_dataclass = args_tree._is_dataclass
        args_tree._is_dataclass = disabled_dataclass

        import dataclasses

        def reverse_dataclass(value):
            if dataclasses.is_dataclass(value):
                return reverse_proxy_class(type(value))(**value)
            else:
                return value

        for name, rsd in state_dict.items():
            output = state_dict[name]["outputs_original"]
            out_tree = args_tree.ArgsTree((output, None), False)
            # dataclass type needs to be reversed to torch type to avoid saving error.
            out = out_tree.map_leaf(reverse_dataclass)
            state_dict[name]["outputs_original"] = out[0]

        args_tree._is_dataclass = original_is_dataclass

        flow.save(state_dict, file_path)



"""A module for registering custom torch2oflow functions and classes."""
import inspect
import importlib.util
from pathlib import Path
from typing import Callable, Dict, List, Optional, Union
from ..import_tools import import_module_from_path
from .manager import transform_mgr
from .builtin_transform import torch2oflow
from ..utils.log_utils import logger

__all__ = ["register"]


def register_torch2oflow_class(cls: type, replacement: type, verbose=True):
    try:
        key = transform_mgr.get_transformed_entity_name(cls)
        transform_mgr.update_class_proxies({key: replacement}, verbose=verbose)

    except Exception as e:
        logger.warning(f"Cannot register {cls=} {replacement=}. {e=}")


def register_torch2oflow_func(func, first_param_type=None, verbose=False):
    if first_param_type is None:
        params = inspect.signature(func).parameters
        first_param_type = params[list(params.keys())[0]].annotation
        if first_param_type == inspect._empty:
            logger.warning(f"Cannot register {func=} {first_param_type=}.")
    try:
        torch2oflow.register(first_param_type)(func)
        logger.debug(f"Register {func=} {first_param_type=}")
        if verbose:
            logger.info(f"Register {func=} {first_param_type=}")
    except Exception as e:
        logger.warning(f"Cannot register {func=} {first_param_type=}. {e=}")


def set_default_registry():
    mocked_packages = transform_mgr.get_mocked_packages()

    def import_module_safely(module_path, module_name):
        nonlocal mocked_packages

        if module_name in mocked_packages:
            return
        try:
            import_module_from_path(module_path)
        except Exception as e:
            logger.warning(f"Failed to import {module_name} from {module_path}. {e=}")

    # compiler_registry_path
    registry_path = Path(__file__).parents[3] / "infer_compiler_registry"

    if importlib.util.find_spec("diffusers") is not None:
        import_module_safely(registry_path / "register_diffusers", "register_diffusers")

    if importlib.util.find_spec("onediff_quant") is not None:
        import_module_safely(
            registry_path / "register_onediff_quant", "register_onediff_quant"
        )

    if importlib.util.find_spec("diffusers_enterprise_lite") is not None:
        import_module_safely(
            registry_path / "register_diffusers_enterprise_lite",
            "register_diffusers_enterprise_lite",
        )


def ensure_list(obj):
    if isinstance(obj, list):
        return obj
    return [obj]


def register(
    *,
    package_names: Optional[List[Union[Path, str]]] = None,
    torch2oflow_class_map: Optional[Dict[type, type]] = None,
    torch2oflow_funcs: Optional[List[Callable]] = None,
):
    if package_names:
        package_names = ensure_list(package_names)
        transform_mgr.load_class_proxies_from_packages(package_names)

    if torch2oflow_class_map:
        for torch_cls, of_cls in torch2oflow_class_map.items():
            register_torch2oflow_class(torch_cls, of_cls)

    if torch2oflow_funcs:
        torch2oflow_funcs = ensure_list(torch2oflow_funcs)
        for func in torch2oflow_funcs:
            register_torch2oflow_func(func)



from diffusers import AutoPipelineForText2Image
from onediff.quantization.quantize_pipeline import QuantPipeline
import argparse 
import torch
from onediff.infer_compiler import oneflow_compile

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--prompt", default="a photo of an astronaut riding a horse on mars")
    parser.add_argument("--height", type= int,default=1024)
    parser.add_argument("--width", type= int, default=1024)
    parser.add_argument("--num_inference_steps", type=int, default=30)
    parser.add_argument("--quantized_model", type=str, required=True)
    return parser.parse_args()

args = parse_args()

pipe = QuantPipeline.from_quantized(
    AutoPipelineForText2Image, args.quantized_model, torch_dtype=torch.float16, variant="fp16", use_safetensors=True
)
pipe = pipe.to("cuda")

pipe_kwargs = dict(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=args.num_inference_steps,
)

pipe.unet = oneflow_compile(pipe.unet)
pipe(**pipe_kwargs).images[0].save("test.png")
